package solutions.bitbadger.documents.groovy

import org.junit.jupiter.api.AfterEach
import org.junit.jupiter.api.DisplayName
import org.junit.jupiter.api.Test
import solutions.bitbadger.documents.Dialect
//import solutions.bitbadger.documents.DocumentException
import solutions.bitbadger.documents.Field
import solutions.bitbadger.documents.FieldFormat
import solutions.bitbadger.documents.Op
import solutions.bitbadger.documents.support.ForceDialect

import static groovy.test.GroovyAssert.*

/**
 * Unit tests for the `Field` class
 */
@DisplayName('JVM | Groovy | Field')
class FieldTest {

    /**
     * Clear the connection string (resets Dialect)
     */
    @AfterEach
    void cleanUp() {
        ForceDialect.none()
    }

    // ~~~ INSTANCE METHODS ~~~

//    TODO: fix java.base open issue
//    @Test
//    @DisplayName('withParameterName fails for invalid name')
//    void withParamNameFails() {
//        assertThrows(DocumentException) { Field.equal('it', '').withParameterName('2424') }
//    }

    @Test
    @DisplayName('withParameterName works with colon prefix')
    void withParamNameColon() {
        def field = Field.equal('abc', '22').withQualifier('me')
        def withParam = field.withParameterName(':test')
        assertNotSame('A new Field instance should have been created', field, withParam)
        assertEquals('Name should have been preserved', field.name, withParam.name)
        assertEquals('Comparison should have been preserved', field.comparison, withParam.comparison)
        assertEquals('Parameter name not set correctly', ':test', withParam.parameterName)
        assertEquals('Qualifier should have been preserved', field.qualifier, withParam.qualifier)
    }

    @Test
    @DisplayName('withParameterName works with at-sign prefix')
    void withParamNameAtSign() {
        def field = Field.equal('def', '44')
        def withParam = field.withParameterName('@unit')
        assertNotSame('A new Field instance should have been created', field, withParam)
        assertEquals('Name should have been preserved', field.name, withParam.name)
        assertEquals('Comparison should have been preserved', field.comparison, withParam.comparison)
        assertEquals('Parameter name not set correctly', '@unit', withParam.parameterName)
        assertEquals('Qualifier should have been preserved', field.qualifier, withParam.qualifier)
    }

    @Test
    @DisplayName('withQualifier sets qualifier correctly')
    void withQualifier() {
        def field = Field.equal('j', 'k')
        def withQual = field.withQualifier('test')
        assertNotSame('A new Field instance should have been created', field, withQual)
        assertEquals('Name should have been preserved', field.name, withQual.name)
        assertEquals('Comparison should have been preserved', field.comparison, withQual.comparison)
        assertEquals('Parameter Name should have been preserved', field.parameterName, withQual.parameterName)
        assertEquals('Qualifier not set correctly', 'test', withQual.qualifier)
    }

    @Test
    @DisplayName('path generates for simple unqualified PostgreSQL field')
    void pathPostgresSimpleUnqualified() {
        assertEquals('Path not correct', "data->>'SomethingCool'",
                Field.greaterOrEqual('SomethingCool', 18).path(Dialect.POSTGRESQL, FieldFormat.SQL))
    }

    @Test
    @DisplayName('path generates for simple qualified PostgreSQL field')
    void pathPostgresSimpleQualified() {
        assertEquals('Path not correct', "this.data->>'SomethingElse'",
                Field.less('SomethingElse', 9).withQualifier('this').path(Dialect.POSTGRESQL, FieldFormat.SQL))
    }

    @Test
    @DisplayName('path generates for nested unqualified PostgreSQL field')
    void pathPostgresNestedUnqualified() {
        assertEquals('Path not correct', "data#>>'{My,Nested,Field}'",
                Field.equal('My.Nested.Field', 'howdy').path(Dialect.POSTGRESQL, FieldFormat.SQL))
    }

    @Test
    @DisplayName('path generates for nested qualified PostgreSQL field')
    void pathPostgresNestedQualified() {
        assertEquals('Path not correct', "bird.data#>>'{Nest,Away}'",
                Field.equal('Nest.Away', 'doc').withQualifier('bird').path(Dialect.POSTGRESQL, FieldFormat.SQL))
    }

    @Test
    @DisplayName('path generates for simple unqualified SQLite field')
    void pathSQLiteSimpleUnqualified() {
        assertEquals('Path not correct', "data->>'SomethingCool'",
                Field.greaterOrEqual('SomethingCool', 18).path(Dialect.SQLITE, FieldFormat.SQL))
    }

    @Test
    @DisplayName('path generates for simple qualified SQLite field')
    void pathSQLiteSimpleQualified() {
        assertEquals('Path not correct', "this.data->>'SomethingElse'",
                Field.less('SomethingElse', 9).withQualifier('this').path(Dialect.SQLITE, FieldFormat.SQL))
    }

    @Test
    @DisplayName('path generates for nested unqualified SQLite field')
    void pathSQLiteNestedUnqualified() {
        assertEquals('Path not correct', "data->'My'->'Nested'->>'Field'",
                Field.equal('My.Nested.Field', 'howdy').path(Dialect.SQLITE, FieldFormat.SQL))
    }

    @Test
    @DisplayName('path generates for nested qualified SQLite field')
    void pathSQLiteNestedQualified() {
        assertEquals('Path not correct', "bird.data->'Nest'->>'Away'",
                Field.equal('Nest.Away', 'doc').withQualifier('bird').path(Dialect.SQLITE, FieldFormat.SQL))
    }

    @Test
    @DisplayName('toWhere generates for exists w/o qualifier | PostgreSQL')
    void toWhereExistsNoQualPostgres() {
        ForceDialect.postgres()
        assertEquals('Field WHERE clause not generated correctly', "data->>'that_field' IS NOT NULL",
                Field.exists('that_field').toWhere())
    }

    @Test
    @DisplayName('toWhere generates for exists w/o qualifier | SQLite')
    void toWhereExistsNoQualSQLite() {
        ForceDialect.sqlite()
        assertEquals('Field WHERE clause not generated correctly', "data->>'that_field' IS NOT NULL",
                Field.exists('that_field').toWhere())
    }

    @Test
    @DisplayName('toWhere generates for not-exists w/o qualifier | PostgreSQL')
    void toWhereNotExistsNoQualPostgres() {
        ForceDialect.postgres()
        assertEquals('Field WHERE clause not generated correctly', "data->>'a_field' IS NULL",
                Field.notExists('a_field').toWhere())
    }

    @Test
    @DisplayName('toWhere generates for not-exists w/o qualifier | SQLite')
    void toWhereNotExistsNoQualSQLite() {
        ForceDialect.sqlite()
        assertEquals('Field WHERE clause not generated correctly', "data->>'a_field' IS NULL",
                Field.notExists('a_field').toWhere())
    }

    @Test
    @DisplayName('toWhere generates for BETWEEN w/o qualifier, numeric range | PostgreSQL')
    void toWhereBetweenNoQualNumericPostgres() {
        ForceDialect.postgres()
        assertEquals('Field WHERE clause not generated correctly',
                "(data->>'age')::numeric BETWEEN @agemin AND @agemax", Field.between('age', 13, 17, '@age').toWhere())
    }

    @Test
    @DisplayName('toWhere generates for BETWEEN w/o qualifier, alphanumeric range | PostgreSQL')
    void toWhereBetweenNoQualAlphaPostgres() {
        ForceDialect.postgres()
        assertEquals('Field WHERE clause not generated correctly', "data->>'city' BETWEEN :citymin AND :citymax",
                Field.between('city', 'Atlanta', 'Chicago', ':city').toWhere())
    }

    @Test
    @DisplayName('toWhere generates for BETWEEN w/o qualifier | SQLite')
    void toWhereBetweenNoQualSQLite() {
        ForceDialect.sqlite()
        assertEquals('Field WHERE clause not generated correctly', "data->>'age' BETWEEN @agemin AND @agemax",
                Field.between('age', 13, 17, '@age').toWhere())
    }

    @Test
    @DisplayName('toWhere generates for BETWEEN w/ qualifier, numeric range | PostgreSQL')
    void toWhereBetweenQualNumericPostgres() {
        ForceDialect.postgres()
        assertEquals('Field WHERE clause not generated correctly',
                "(test.data->>'age')::numeric BETWEEN @agemin AND @agemax",
                Field.between('age', 13, 17, '@age').withQualifier('test').toWhere())
    }

    @Test
    @DisplayName('toWhere generates for BETWEEN w/ qualifier, alphanumeric range | PostgreSQL')
    void toWhereBetweenQualAlphaPostgres() {
        ForceDialect.postgres()
        assertEquals('Field WHERE clause not generated correctly', "unit.data->>'city' BETWEEN :citymin AND :citymax",
                Field.between('city', 'Atlanta', 'Chicago', ':city').withQualifier('unit').toWhere())
    }

    @Test
    @DisplayName('toWhere generates for BETWEEN w/ qualifier | SQLite')
    void toWhereBetweenQualSQLite() {
        ForceDialect.sqlite()
        assertEquals('Field WHERE clause not generated correctly', "my.data->>'age' BETWEEN @agemin AND @agemax",
                Field.between('age', 13, 17, '@age').withQualifier('my').toWhere())
    }

    @Test
    @DisplayName('toWhere generates for IN/any, numeric values | PostgreSQL')
    void toWhereAnyNumericPostgres() {
        ForceDialect.postgres()
        assertEquals('Field WHERE clause not generated correctly',
                "(data->>'even')::numeric IN (:nbr_0, :nbr_1, :nbr_2)",
                Field.any('even', List.of(2, 4, 6), ':nbr').toWhere())
    }

    @Test
    @DisplayName('toWhere generates for IN/any, alphanumeric values | PostgreSQL')
    void toWhereAnyAlphaPostgres() {
        ForceDialect.postgres()
        assertEquals('Field WHERE clause not generated correctly', "data->>'test' IN (:city_0, :city_1)",
                Field.any('test', List.of('Atlanta', 'Chicago'), ':city').toWhere())
    }

    @Test
    @DisplayName('toWhere generates for IN/any | SQLite')
    void toWhereAnySQLite() {
        ForceDialect.sqlite()
        assertEquals('Field WHERE clause not generated correctly', "data->>'test' IN (:city_0, :city_1)",
                Field.any('test', List.of('Atlanta', 'Chicago'), ':city').toWhere())
    }

    @Test
    @DisplayName('toWhere generates for inArray | PostgreSQL')
    void toWhereInArrayPostgres() {
        ForceDialect.postgres()
        assertEquals('Field WHERE clause not generated correctly', "data->'even' ??| ARRAY[:it_0, :it_1, :it_2, :it_3]",
                Field.inArray('even', 'tbl', List.of(2, 4, 6, 8), ':it').toWhere())
    }

    @Test
    @DisplayName('toWhere generates for inArray | SQLite')
    void toWhereInArraySQLite() {
        ForceDialect.sqlite()
        assertEquals('Field WHERE clause not generated correctly',
                "EXISTS (SELECT 1 FROM json_each(tbl.data, '\$.test') WHERE value IN (:city_0, :city_1))",
                Field.inArray('test', 'tbl', List.of('Atlanta', 'Chicago'), ':city').toWhere())
    }

    @Test
    @DisplayName('toWhere generates for others w/o qualifier | PostgreSQL')
    void toWhereOtherNoQualPostgres() {
        ForceDialect.postgres()
        assertEquals('Field WHERE clause not generated correctly', "data->>'some_field' = :value",
                Field.equal('some_field', '', ':value').toWhere())
    }

    @Test
    @DisplayName('toWhere generates for others w/o qualifier | SQLite')
    void toWhereOtherNoQualSQLite() {
        ForceDialect.sqlite()
        assertEquals('Field WHERE clause not generated correctly', "data->>'some_field' = :value",
                Field.equal('some_field', '', ':value').toWhere())
    }

    @Test
    @DisplayName('toWhere generates no-parameter w/ qualifier | PostgreSQL')
    void toWhereNoParamWithQualPostgres() {
        ForceDialect.postgres()
        assertEquals('Field WHERE clause not generated correctly', "test.data->>'no_field' IS NOT NULL",
                Field.exists('no_field').withQualifier('test').toWhere())
    }

    @Test
    @DisplayName('toWhere generates no-parameter w/ qualifier | SQLite')
    void toWhereNoParamWithQualSQLite() {
        ForceDialect.sqlite()
        assertEquals('Field WHERE clause not generated correctly', "test.data->>'no_field' IS NOT NULL",
                Field.exists('no_field').withQualifier('test').toWhere())
    }

    @Test
    @DisplayName('toWhere generates parameter w/ qualifier | PostgreSQL')
    void toWhereParamWithQualPostgres() {
        ForceDialect.postgres()
        assertEquals('Field WHERE clause not generated correctly', "(q.data->>'le_field')::numeric <= :it",
                Field.lessOrEqual('le_field', 18, ':it').withQualifier('q').toWhere())
    }

    @Test
    @DisplayName('toWhere generates parameter w/ qualifier | SQLite')
    void toWhereParamWithQualSQLite() {
        ForceDialect.sqlite()
        assertEquals('Field WHERE clause not generated correctly', "q.data->>'le_field' <= :it",
                Field.lessOrEqual('le_field', 18, ':it').withQualifier('q').toWhere())
    }

    // ~~~ STATIC CONSTRUCTOR TESTS ~~~

    @Test
    @DisplayName('equal constructs a field w/o parameter name')
    void equalCtor() {
        def field = Field.equal('Test', 14)
        assertEquals('Field name not filled correctly', 'Test', field.name)
        assertEquals('Field comparison operation not filled correctly', Op.EQUAL, field.comparison.op)
        assertEquals('Field comparison value not filled correctly', 14, field.comparison.value)
        assertNull('The parameter name should have been null', field.parameterName)
        assertNull('The qualifier should have been null', field.qualifier)
    }

    @Test
    @DisplayName('equal constructs a field w/ parameter name')
    void equalParameterCtor() {
        def field = Field.equal('Test', 14, ':w')
        assertEquals('Field name not filled correctly', 'Test', field.name)
        assertEquals('Field comparison operation not filled correctly', Op.EQUAL, field.comparison.op)
        assertEquals('Field comparison value not filled correctly', 14, field.comparison.value)
        assertEquals('Field parameter name not filled correctly', ':w', field.parameterName)
        assertNull('The qualifier should have been null', field.qualifier)
    }

    @Test
    @DisplayName('greater constructs a field w/o parameter name')
    void greaterCtor() {
        def field = Field.greater('Great', 'night')
        assertEquals('Field name not filled correctly', 'Great', field.name)
        assertEquals('Field comparison operation not filled correctly', Op.GREATER, field.comparison.op)
        assertEquals('Field comparison value not filled correctly', 'night', field.comparison.value)
        assertNull('The parameter name should have been null', field.parameterName)
        assertNull('The qualifier should have been null', field.qualifier)
    }

    @Test
    @DisplayName('greater constructs a field w/ parameter name')
    void greaterParameterCtor() {
        def field = Field.greater('Great', 'night', ':yeah')
        assertEquals('Field name not filled correctly', 'Great', field.name)
        assertEquals('Field comparison operation not filled correctly', Op.GREATER, field.comparison.op)
        assertEquals('Field comparison value not filled correctly', 'night', field.comparison.value)
        assertEquals('Field parameter name not filled correctly', ':yeah', field.parameterName)
        assertNull('The qualifier should have been null', field.qualifier)
    }

    @Test
    @DisplayName('greaterOrEqual constructs a field w/o parameter name')
    void greaterOrEqualCtor() {
        def field = Field.greaterOrEqual('Nice', 88L)
        assertEquals('Field name not filled correctly', 'Nice', field.name)
        assertEquals('Field comparison operation not filled correctly', Op.GREATER_OR_EQUAL, field.comparison.op)
        assertEquals('Field comparison value not filled correctly', 88L, field.comparison.value)
        assertNull('The parameter name should have been null', field.parameterName)
        assertNull('The qualifier should have been null', field.qualifier)
    }

    @Test
    @DisplayName('greaterOrEqual constructs a field w/ parameter name')
    void greaterOrEqualParameterCtor() {
        def field = Field.greaterOrEqual('Nice', 88L, ':nice')
        assertEquals('Field name not filled correctly', 'Nice', field.name)
        assertEquals('Field comparison operation not filled correctly', Op.GREATER_OR_EQUAL, field.comparison.op)
        assertEquals('Field comparison value not filled correctly', 88L, field.comparison.value)
        assertEquals('Field parameter name not filled correctly', ':nice', field.parameterName)
        assertNull('The qualifier should have been null', field.qualifier)
    }

    @Test
    @DisplayName('less constructs a field w/o parameter name')
    void lessCtor() {
        def field = Field.less('Lesser', 'seven')
        assertEquals('Field name not filled correctly', 'Lesser', field.name)
        assertEquals('Field comparison operation not filled correctly', Op.LESS, field.comparison.op)
        assertEquals('Field comparison value not filled correctly', 'seven', field.comparison.value)
        assertNull('The parameter name should have been null', field.parameterName)
        assertNull('The qualifier should have been null', field.qualifier)
    }

    @Test
    @DisplayName('less constructs a field w/ parameter name')
    void lessParameterCtor() {
        def field = Field.less('Lesser', 'seven', ':max')
        assertEquals('Field name not filled correctly', 'Lesser', field.name)
        assertEquals('Field comparison operation not filled correctly', Op.LESS, field.comparison.op)
        assertEquals('Field comparison value not filled correctly', 'seven', field.comparison.value)
        assertEquals('Field parameter name not filled correctly', ':max', field.parameterName)
        assertNull('The qualifier should have been null', field.qualifier)
    }

    @Test
    @DisplayName('lessOrEqual constructs a field w/o parameter name')
    void lessOrEqualCtor() {
        def field = Field.lessOrEqual('Nobody', 'KNOWS')
        assertEquals('Field name not filled correctly', 'Nobody', field.name)
        assertEquals('Field comparison operation not filled correctly', Op.LESS_OR_EQUAL, field.comparison.op)
        assertEquals('Field comparison value not filled correctly', 'KNOWS', field.comparison.value)
        assertNull('The parameter name should have been null', field.parameterName)
        assertNull('The qualifier should have been null', field.qualifier)
    }

    @Test
    @DisplayName('lessOrEqual constructs a field w/ parameter name')
    void lessOrEqualParameterCtor() {
        def field = Field.lessOrEqual('Nobody', 'KNOWS', ':nope')
        assertEquals('Field name not filled correctly', 'Nobody', field.name)
        assertEquals('Field comparison operation not filled correctly', Op.LESS_OR_EQUAL, field.comparison.op)
        assertEquals('Field comparison value not filled correctly', 'KNOWS', field.comparison.value)
        assertEquals('Field parameter name not filled correctly', ':nope', field.parameterName)
        assertNull('The qualifier should have been null', field.qualifier)
    }

    @Test
    @DisplayName('notEqual constructs a field w/o parameter name')
    void notEqualCtor() {
        def field = Field.notEqual('Park', 'here')
        assertEquals('Field name not filled correctly', 'Park', field.name)
        assertEquals('Field comparison operation not filled correctly', Op.NOT_EQUAL, field.comparison.op)
        assertEquals('Field comparison value not filled correctly', 'here', field.comparison.value)
        assertNull('The parameter name should have been null', field.parameterName)
        assertNull('The qualifier should have been null', field.qualifier)
    }

    @Test
    @DisplayName('notEqual constructs a field w/ parameter name')
    void notEqualParameterCtor() {
        def field = Field.notEqual('Park', 'here', ':now')
        assertEquals('Field name not filled correctly', 'Park', field.name)
        assertEquals('Field comparison operation not filled correctly', Op.NOT_EQUAL, field.comparison.op)
        assertEquals('Field comparison value not filled correctly', 'here', field.comparison.value)
        assertEquals('Field parameter name not filled correctly', ':now', field.parameterName)
        assertNull('The qualifier should have been null', field.qualifier)
    }

    @Test
    @DisplayName('between constructs a field w/o parameter name')
    void betweenCtor() {
        def field = Field.between('Age', 18, 49)
        assertEquals('Field name not filled correctly', 'Age', field.name)
        assertEquals('Field comparison operation not filled correctly', Op.BETWEEN, field.comparison.op)
        assertEquals('Field comparison min value not filled correctly', 18, field.comparison.value.first)
        assertEquals('Field comparison max value not filled correctly', 49, field.comparison.value.second, )
        assertNull('The parameter name should have been null', field.parameterName)
        assertNull('The qualifier should have been null', field.qualifier)
    }

    @Test
    @DisplayName('between constructs a field w/ parameter name')
    void betweenParameterCtor() {
        def field = Field.between('Age', 18, 49, ':limit')
        assertEquals('Field name not filled correctly', 'Age', field.name)
        assertEquals('Field comparison operation not filled correctly', Op.BETWEEN, field.comparison.op)
        assertEquals('Field comparison min value not filled correctly', 18, field.comparison.value.first)
        assertEquals('Field comparison max value not filled correctly', 49, field.comparison.value.second)
        assertEquals('Field parameter name not filled correctly', ':limit', field.parameterName)
        assertNull('The qualifier should have been null', field.qualifier)
    }

    @Test
    @DisplayName('any constructs a field w/o parameter name')
    void anyCtor() {
        def field = Field.any('Here', List.of(8, 16, 32))
        assertEquals('Field name not filled correctly', 'Here', field.name)
        assertEquals('Field comparison operation not filled correctly', Op.IN, field.comparison.op)
        assertEquals('Field comparison value not filled correctly', List.of(8, 16, 32), field.comparison.value)
        assertNull('The parameter name should have been null', field.parameterName)
        assertNull('The qualifier should have been null', field.qualifier)
    }

    @Test
    @DisplayName('any constructs a field w/ parameter name')
    void anyParameterCtor() {
        def field = Field.any('Here', List.of(8, 16, 32), ':list')
        assertEquals('Field name not filled correctly', 'Here', field.name)
        assertEquals('Field comparison operation not filled correctly', Op.IN, field.comparison.op)
        assertEquals('Field comparison value not filled correctly', List.of(8, 16, 32), field.comparison.value)
        assertEquals('Field parameter name not filled correctly', ':list', field.parameterName)
        assertNull('The qualifier should have been null', field.qualifier)
    }

    @Test
    @DisplayName('inArray constructs a field w/o parameter name')
    void inArrayCtor() {
        def field = Field.inArray('ArrayField', 'table', List.of('z'))
        assertEquals('Field name not filled correctly', 'ArrayField', field.name)
        assertEquals('Field comparison operation not filled correctly', Op.IN_ARRAY, field.comparison.op)
        assertEquals('Field comparison table not filled correctly', 'table', field.comparison.value.first)
        assertEquals('Field comparison values not filled correctly', List.of('z'), field.comparison.value.second)
        assertNull('The parameter name should have been null', field.parameterName)
        assertNull('The qualifier should have been null', field.qualifier)
    }

    @Test
    @DisplayName('inArray constructs a field w/ parameter name')
    void inArrayParameterCtor() {
        def field = Field.inArray('ArrayField', 'table', List.of('z'), ':a')
        assertEquals('Field name not filled correctly', 'ArrayField', field.name)
        assertEquals('Field comparison operation not filled correctly', Op.IN_ARRAY, field.comparison.op)
        assertEquals('Field comparison table not filled correctly', 'table', field.comparison.value.first)
        assertEquals('Field comparison values not filled correctly', List.of('z'), field.comparison.value.second)
        assertEquals('Field parameter name not filled correctly', ':a', field.parameterName)
        assertNull('The qualifier should have been null', field.qualifier)
    }

    @Test
    @DisplayName('exists constructs a field')
    void existsCtor() {
        def field = Field.exists 'Groovy'
        assertEquals('Field name not filled correctly', 'Groovy', field.name)
        assertEquals('Field comparison operation not filled correctly', Op.EXISTS, field.comparison.op)
        assertEquals('Field comparison value not filled correctly', '', field.comparison.value)
        assertNull('The parameter name should have been null', field.parameterName)
        assertNull('The qualifier should have been null', field.qualifier)
    }

    @Test
    @DisplayName('notExists constructs a field')
    void notExistsCtor() {
        def field = Field.notExists 'Groovy'
        assertEquals('Field name not filled correctly', 'Groovy', field.name)
        assertEquals('Field comparison operation not filled correctly', Op.NOT_EXISTS, field.comparison.op)
        assertEquals('Field comparison value not filled correctly', '', field.comparison.value)
        assertNull('The parameter name should have been null', field.parameterName)
        assertNull('The qualifier should have been null', field.qualifier)
    }

    @Test
    @DisplayName('named constructs a field')
    void namedCtor() {
        def field = Field.named('Tacos')
        assertEquals('Field name not filled correctly', 'Tacos', field.name)
        assertEquals('Field comparison operation not filled correctly', Op.EQUAL, field.comparison.op)
        assertEquals('Field comparison value not filled correctly', '', field.comparison.value)
        assertNull('The parameter name should have been null', field.parameterName)
        assertNull('The qualifier should have been null', field.qualifier)
    }

//    TODO: fix java.base open issue
//    @Test
//    @DisplayName('static constructors fail for invalid parameter name')
//    void staticCtorsFailOnParamName() {
//        assertThrows(DocumentException) { Field.equal('a', 'b', "that ain't it, Jack...") }
//    }

    @Test
    @DisplayName('nameToPath creates a simple PostgreSQL SQL name')
    void nameToPathPostgresSimpleSQL() {
        assertEquals('Path not constructed correctly', "data->>'Simple'",
                Field.nameToPath('Simple', Dialect.POSTGRESQL, FieldFormat.SQL))
    }

    @Test
    @DisplayName('nameToPath creates a simple SQLite SQL name')
    void nameToPathSQLiteSimpleSQL() {
        assertEquals('Path not constructed correctly', "data->>'Simple'",
                Field.nameToPath('Simple', Dialect.SQLITE, FieldFormat.SQL))
    }

    @Test
    @DisplayName('nameToPath creates a nested PostgreSQL SQL name')
    void nameToPathPostgresNestedSQL() {
        assertEquals('Path not constructed correctly', "data#>>'{A,Long,Path,to,the,Property}'",
                Field.nameToPath('A.Long.Path.to.the.Property', Dialect.POSTGRESQL, FieldFormat.SQL))
    }

    @Test
    @DisplayName('nameToPath creates a nested SQLite SQL name')
    void nameToPathSQLiteNestedSQL() {
        assertEquals('Path not constructed correctly', "data->'A'->'Long'->'Path'->'to'->'the'->>'Property'",
                Field.nameToPath('A.Long.Path.to.the.Property', Dialect.SQLITE, FieldFormat.SQL))
    }

    @Test
    @DisplayName('nameToPath creates a simple PostgreSQL JSON name')
    void nameToPathPostgresSimpleJSON() {
        assertEquals('Path not constructed correctly', "data->'Simple'",
                Field.nameToPath('Simple', Dialect.POSTGRESQL, FieldFormat.JSON))
    }

    @Test
    @DisplayName('nameToPath creates a simple SQLite JSON name')
    void nameToPathSQLiteSimpleJSON() {
        assertEquals('Path not constructed correctly', "data->'Simple'",
                Field.nameToPath('Simple', Dialect.SQLITE, FieldFormat.JSON))
    }

    @Test
    @DisplayName('nameToPath creates a nested PostgreSQL JSON name')
    void nameToPathPostgresNestedJSON() {
        assertEquals('Path not constructed correctly', "data#>'{A,Long,Path,to,the,Property}'",
                Field.nameToPath('A.Long.Path.to.the.Property', Dialect.POSTGRESQL, FieldFormat.JSON))
    }

    @Test
    @DisplayName('nameToPath creates a nested SQLite JSON name')
    void nameToPathSQLiteNestedJSON() {
        assertEquals('Path not constructed correctly', "data->'A'->'Long'->'Path'->'to'->'the'->'Property'",
                Field.nameToPath('A.Long.Path.to.the.Property', Dialect.SQLITE, FieldFormat.JSON))
    }
}
