package solutions.bitbadger.documents.groovy.query

import org.junit.jupiter.api.AfterEach
import org.junit.jupiter.api.DisplayName
import org.junit.jupiter.api.Test
import solutions.bitbadger.documents.AutoId
import solutions.bitbadger.documents.Configuration
//import solutions.bitbadger.documents.DocumentException
import solutions.bitbadger.documents.query.DocumentQuery
import solutions.bitbadger.documents.support.ForceDialect

import static solutions.bitbadger.documents.support.TypesKt.TEST_TABLE
import static groovy.test.GroovyAssert.*

/**
 * Unit tests for the `Document` object
 */
@DisplayName('JVM | Groovy | Query | DocumentQuery')
class DocumentQueryTest {

    /**
     * Clear the connection string (resets Dialect)
     */
    @AfterEach
    void cleanUp() {
        ForceDialect.none()
    }

    @Test
    @DisplayName('insert generates no auto ID | PostgreSQL')
    void insertNoAutoPostgres() {
        ForceDialect.postgres()
        assertEquals("INSERT INTO $TEST_TABLE VALUES (:data)".toString(), DocumentQuery.insert(TEST_TABLE))
    }

    @Test
    @DisplayName('insert generates no auto ID | SQLite')
    void insertNoAutoSQLite() {
        ForceDialect.sqlite()
        assertEquals("INSERT INTO $TEST_TABLE VALUES (:data)".toString(), DocumentQuery.insert(TEST_TABLE))
    }

    @Test
    @DisplayName('insert generates auto number | PostgreSQL')
    void insertAutoNumberPostgres() {
        ForceDialect.postgres()
        assertEquals("INSERT INTO $TEST_TABLE VALUES (:data::jsonb || ('{\"id\":' || (SELECT ".toString() +
                        "COALESCE(MAX((data->>'id')::numeric), 0) + 1 FROM $TEST_TABLE) || '}')::jsonb)".toString(),
                DocumentQuery.insert(TEST_TABLE, AutoId.NUMBER))
    }

    @Test
    @DisplayName('insert generates auto number | SQLite')
    void insertAutoNumberSQLite() {
        ForceDialect.sqlite()
        assertEquals("INSERT INTO $TEST_TABLE VALUES (json_set(:data, '\$.id', ".toString() +
                        "(SELECT coalesce(max(data->>'id'), 0) + 1 FROM $TEST_TABLE)))".toString(),
                DocumentQuery.insert(TEST_TABLE, AutoId.NUMBER))
    }

    @Test
    @DisplayName('insert generates auto UUID | PostgreSQL')
    void insertAutoUUIDPostgres() {
        ForceDialect.postgres()
        def query = DocumentQuery.insert(TEST_TABLE, AutoId.UUID)
        assertTrue("Query start not correct (actual: $query)",
                query.startsWith("INSERT INTO $TEST_TABLE VALUES (:data::jsonb || '{\"id\":\""))
        assertTrue('Query end not correct', query.endsWith("\"}')"))
    }

    @Test
    @DisplayName('insert generates auto UUID | SQLite')
    void insertAutoUUIDSQLite() {
        ForceDialect.sqlite()
        def query = DocumentQuery.insert(TEST_TABLE, AutoId.UUID)
        assertTrue("Query start not correct (actual: $query)",
                query.startsWith("INSERT INTO $TEST_TABLE VALUES (json_set(:data, '\$.id', '"))
        assertTrue('Query end not correct', query.endsWith("'))"))
    }

    @Test
    @DisplayName('insert generates auto random string | PostgreSQL')
    void insertAutoRandomPostgres() {
        try {
            ForceDialect.postgres()
            Configuration.idStringLength = 8
            def query = DocumentQuery.insert(TEST_TABLE, AutoId.RANDOM_STRING)
            assertTrue("Query start not correct (actual: $query)",
                    query.startsWith("INSERT INTO $TEST_TABLE VALUES (:data::jsonb || '{\"id\":\""))
            assertTrue('Query end not correct', query.endsWith("\"}')"))
            assertEquals('Random string length incorrect', 8,
                    query.replace("INSERT INTO $TEST_TABLE VALUES (:data::jsonb || '{\"id\":\"", '')
                            .replace("\"}')", '').length())
        } finally {
            Configuration.idStringLength = 16
        }
    }

    @Test
    @DisplayName('insert generates auto random string | SQLite')
    void insertAutoRandomSQLite() {
        ForceDialect.sqlite()
        def query = DocumentQuery.insert(TEST_TABLE, AutoId.RANDOM_STRING)
        assertTrue("Query start not correct (actual: $query)",
                query.startsWith("INSERT INTO $TEST_TABLE VALUES (json_set(:data, '\$.id', '"))
        assertTrue('Query end not correct', query.endsWith("'))"))
        assertEquals('Random string length incorrect', Configuration.idStringLength,
                query.replace("INSERT INTO $TEST_TABLE VALUES (json_set(:data, '\$.id', '", '').replace("'))", '')
                        .length())
    }

//    TODO: resolve java.base open issue
//    @Test
//    @DisplayName('insert fails when no dialect is set')
//    void insertFailsUnknown() {
//        assertThrows(DocumentException) { DocumentQuery.insert(TEST_TABLE) }
//    }

    @Test
    @DisplayName('save generates correctly')
    void save() {
        ForceDialect.postgres()
        assertEquals('INSERT ON CONFLICT UPDATE statement not constructed correctly',
                "INSERT INTO $TEST_TABLE VALUES (:data) ON CONFLICT ((data->>'id')) DO UPDATE SET data = EXCLUDED.data"
                        .toString(),
                DocumentQuery.save(TEST_TABLE))
    }

    @Test
    @DisplayName('update generates successfully')
    void update() {
        assertEquals('Update query not constructed correctly', "UPDATE $TEST_TABLE SET data = :data".toString(),
                DocumentQuery.update(TEST_TABLE))
    }
}
